use crate::{Context, Handler};
use axum::{extract::State, http::StatusCode, Json};
use serde::Deserialize;
use verax::{AccountId, AnyAmount, ReplayProtection, Status, Tag};

#[derive(Deserialize)]
pub struct Deposit {
    pub account: AccountId,
    #[serde(flatten)]
    pub amount: AnyAmount,
    pub memo: String,
    pub tags: Vec<Tag>,
    pub status: Status,
    pub replay_protection: Option<ReplayProtection>,
}

#[async_trait::async_trait]
impl Handler for Deposit {
    type Ok = verax::Transaction;
    type Err = verax::Error;

    async fn handle(self, ctx: &Context) -> Result<Self::Ok, Self::Err> {
        ctx.ledger
            .deposit(
                &self.account,
                self.amount.try_into()?,
                self.status,
                self.tags,
                self.memo,
                self.replay_protection,
            )
            .await
    }
}

pub async fn handler(
    State(ledger): State<Context>,
    Json(item): Json<Deposit>,
) -> Result<Json<verax::Transaction>, (StatusCode, String)> {
    Ok(Json(item.handle(&ledger).await.map_err(|err| {
        (
            StatusCode::BAD_REQUEST,
            serde_json::to_string(&err).unwrap_or_default(),
        )
    })?))
}
